// NOTE: r in the root corresponds to identity
// FIXME: totally wrong for r != 2 (omega = -1 hardcoded)

#include <iostream>
#include <map>
#include <vector>
#include <stdio.h>

#include "rtree.h"
#include "sparse.h"
#include "conjclasses.h"

template<class T> ostream& dump_thing(const vector<T>& t, ostream& os)
{
  for (typename vector<T>::const_iterator i = t.begin(); i != t.end(); ++i)
    os << *i << " ";
  os << endl;
  return os;
}

typedef sparse_matrix<short> mat;

short omega = -1;

// gen -1 is the identity

// compute rho(z)
inline mat cyclic_gen_rep(int rep)
{
  if (rep == 0)
    return mat(1);
  else if (rep == 1)
    return mat(omega);
  else
    {
      assert(0);
      return mat();
    }
}

typedef map<pair<rtree*,int>,mat> cache_t;
cache_t rep_cache;

mat compute_rep(rtree& rt, int gen)
{
  cache_t::iterator it = rep_cache.find(make_pair(&rt, gen));
  if (it != rep_cache.end())
    return it->second;

#define RETURN(m) return (rep_cache[make_pair(&rt, gen)] = (m))
#define BARF return (assert(0),mat())

  int r = rt.r;
  int height = rt.get_height();
  assert(gen <= height);
#if 0
  if (gen == -1)
    RETURN(idn(dim));
  assert(gen >= 0);
#endif
  if (height == 0)
    {
      if (gen == 0)
	RETURN(cyclic_gen_rep(rt.root));
      else if (gen == -1)
	RETURN(mat(1));
      else
	BARF;
    }
  assert(height > 0);
  int d = rt.get_d();
  if (gen == height)
    {
      mat m = compute_rep(*(rt.kids[0]), -1);
      for (unsigned i = 1; i < rt.kids.size(); i++)
	m = mat::tensor(m, compute_rep(*(rt.kids[i]), -1));
      if (d == 2)
	{
	  RETURN(mat::tensor(m, cyclic_gen_rep(rt.root / (r/d))));
	}
      else if (d == 1)
	{
	  mat m1;
	  m1.insert(m, m.get_rows() + 1, 1);
	  m1.insert(m, 1, m.get_cols() + 1);
	  RETURN(m1);
	}
    }
  else
    {
      mat m = compute_rep(*(rt.kids[0]), gen);
      for (unsigned int i = 1; i < rt.kids.size(); i++)
	m = mat::tensor(m, compute_rep(*(rt.kids[i]), -1));
      if (d == 2)
	RETURN(m);
      else if (d == 1)
	{
	  mat m1;
	  m1.insert(mat::tensor(compute_rep(*(rt.kids[0]), gen),
				compute_rep(*(rt.kids[1]), -1)),
		    1,1);

	  m1.insert(mat::tensor(compute_rep(*(rt.kids[0]), -1),
				compute_rep(*(rt.kids[1]), gen)),
		    m.get_rows()+1, m.get_cols()+1);
	  RETURN(m1);
	}
      else
	BARF;
    }
  BARF;
#undef RETURN
#undef BARF
}


// should really be const, ugh
mat eval_rep_at_class(rtree& repn, const string& class_rep) {
  mat m = compute_rep(repn, -1);
  for (unsigned i = 0; i < class_rep.size(); i++)
    m *= compute_rep(repn, class_rep[i] - 'A');
  return m;
}

struct character {
  vector<int> val; // value at each conjugacy class
  int dim;
};

vector<bint> class_sizes;
vector<character> chartab;
unsigned nrepns;
bint groupsize;

void build_character_table(vector<rtree>& rtrees) {
  cout << "Building character table..." << endl;
  chartab.clear();
  unsigned n = nrepns = rtrees.size();
  groupsize = 0;
  for (unsigned i = 0; i < n; i++) {
    cerr << "Done " << i << " of " << n << "         \r";
    character c;
    c.dim = rtrees[i].get_dim().get_ui();
    class_sizes.push_back(class_size(rtrees[i]));
    for (unsigned j = 0; j < n; j++) {
      mat m = eval_rep_at_class(rtrees[i], class_rep(rtrees[j]));
      c.val.push_back(m.trace());
    }
    assert(c.val.size() == nrepns);
    chartab.push_back(c);
    groupsize += c.dim * c.dim;
  }
  assert(chartab.size() == nrepns);
  cerr << endl;
}

inline int get_char(int character, int conjclass) {
  return chartab[character].val[conjclass];
}

void dump_char_table(void)
{
  printf("      | ");
  for (unsigned i = 0; i < nrepns; i++)
    printf("%2u ", i);
  printf("\n");
  printf("      | ");
  for (unsigned i = 0; i < nrepns; i++)
    printf("%2ld ", class_sizes[i].get_ui());
  printf("\n");
  for (unsigned i = 0; i < nrepns; i++) {
    printf("%2u %2d | ", i, chartab[i].dim);
    for (unsigned j = 0; j < nrepns; j++) {
      printf("%2d ", chartab[i].val[j]);
    }
    printf("\n");
  }
  printf("\n");
}


int compute_dimlcm(void) {
  int dl = 1;
  for (unsigned i = 0; i < nrepns; i++)
    dl = lcm(dl, chartab[i].dim);
  return dl;
}
    

void dump_eigenvalues(void)
{
  int dl = compute_dimlcm();
  printf("dim|  (lcm = %d)\n",dl);
  for (unsigned i = 0; i < nrepns; i++) {
    printf("%2d | ", chartab[i].dim);
    for (unsigned j = 0; j < nrepns; j++) {
      printf("%2d ", chartab[i].val[j] * dl / chartab[i].dim);
    }
    printf("\n");
  }
  printf("\n");
}


bint make_inner_product(unsigned x, unsigned y) {
  bint sum = 0;
  for (unsigned i = 0; i < nrepns; i++) {
    // should be a complex conjugate here
    sum += get_char(x,i) * get_char(y, i) * class_sizes[i];
  }
  cout << "x = " << x << ", y = " << y << ", sum = " << sum 
       << ", groupsize = " << groupsize;
  if (sum % groupsize != 0)
    cout << " (ARGH!!!!)";
  cout << endl;
      
  //assert(sum % groupsize == 0);
  return sum / groupsize;
}


// this is bogus
bool is_orthonormal (void) {
  bool ok = true;
  for (unsigned x = 0; x < nrepns; x++)
    for (unsigned y = 0; y < nrepns; y++) {
      bint ip = make_inner_product(x,y);
      if ((x == y && ip != 1) || (x != y && ip != 0)) {
	cout << "PROBLEM: inner product of " << x << 
	  " with " << y << " is " << ip << endl;
	ok = false;
      }
    }
  return ok;
}

int main(int argc, char *argv[])
{
#if 0
  mat m1 = mat::make_idn(2);
  mat m2 = mat::make_idn(1);
  m1.dump_nice();
  m2.dump_nice();
  mat m3 = mat::tensor(m1, m2);
  m3.dump_nice();

  return 0;
#endif

  assert(argc >= 3);
  int n = atoi(argv[1]);
  int r = atoi(argv[2]);
  assert(r == 2);
  generate_all_rtrees(n-1, r);
  bint csz_total = 0;
  vector<rtree>& treelist = all_rtrees[n-1]; 

  //  vector<chartab> chars;
  for (vector<rtree>::iterator i = treelist.begin();
       i != treelist.end(); ++i)
    {
#if 0
      if (n == 2 && r == 2) {
	chartab c = characters(*i);
	cout << "Characters: ";
	dump_thing(c, cout);
	cout << endl;
	chars.push_back(c);
      }
#endif

      cout << "Tree " << i - treelist.begin() << ":" << endl;
      i->print(cout);
      cout << endl;
      if (isprime(r)) {
	bint csz = class_size(*i);
	csz_total += csz;
	cout << "Conjugacy class rep: " << class_rep(*i) << endl;
	cout << "Conjugacy class has order " << csz << endl;
      }
      cout << endl;

      for (int gen = -1; gen < n; gen++)
	{
	  cout << "At generator " << gen << ":" << endl;
	  mat m = compute_rep(*i, gen);
	  assert(m.square());
	  m.print_nicely(cout);
	  cout << "dim = " << m.get_rows() << ", should be " << i->get_dim();
	  if (m.get_rows() != i->get_dim())
	    cout << " (BAD!)";
	  cout << endl;
	  int det = m.det();
	  if (det == 0)
	    cout << "Singular, BAD!" << endl;
	  cout << endl;
	}
      cout << "=======================================" << endl;

    }
  cout << "Conjugacy class sizes sum to " << csz_total << endl;
  if (csz_total == wr_order(n,r))
    cout << "This is as it should be." << endl;
  else
    cout << "Uh oh, that should be " << wr_order(n,r) << endl;
  //  if (n == 2 && r == 2)
  //  cout << "Characters are orthonormal: " << orthonormal(chars) << endl;
  build_character_table(treelist);
  cout << "Character table:" << endl;
  dump_char_table();
  cout << "Eigenvalues:" << endl;
  dump_eigenvalues();
  if (is_orthonormal()) cout << "Orthonormal! Sweet!" << endl;
  return 0;
}

void never_call_me (mat a)
{
  a.print_nicely(cerr);
  a.dumpme();
  a.dump_nice();
}
