#include <bitset>
#include <vector>
#include <iostream>
#include <stdio.h>
#include <algorithm>
#include <sys/time.h>

#if (__GNUC__ < 3)
#error g++ 2.x miscompiles bitset!
#endif

// #define QUIET

using namespace std;

// Due to limitations of the bitset class, the size of the character
// table must be defined at compile time.

#define CLASSES 5
#define REPS 5

#define CHOOSE2(n) (((n)*((n)-1))/2)

int table[REPS][CLASSES];

typedef bitset<CHOOSE2(REPS)> bs;

bs seps[CLASSES];

int get_int(istream& is) {
  while (is) {
    char c;
    is.get(c);
    switch (c) {
    case '0' ... '9' :
    case '+':
    case '-':
    case '.':
      is.putback(c);
      goto done;
      break;
    default:
      break;
    }
  }
 done:
  int i;
  is >> i;
  return i;
}

void get_table (void) {
  for (int i = 0; i < REPS; i++)
    for (int j = 0; j < CLASSES; j++)
      table[i][j] = get_int(cin);
}

int gcd(int a, int b) {
  if (b == 0)
    return a;
  else
    return gcd(b, a % b);
}

int lcm(int a, int b) {
  return a * b / gcd(a,b);
}

int get_lcm (void) {
  int k = 1;
  for (int i = 0; i < REPS; i++)
    k = lcm(k,table[i][0]);
  return k;
}

void make_eigenvalues (void) {
  int k = get_lcm();
#ifndef QUIET
  printf("LCM = %d\n", k);
#endif
  for (int i = 0; i < REPS; i++) {
    int dim = table[i][0];
    assert(k % dim == 0);
    for (int j = 0; j < CLASSES; j++) {
      table[i][j] *= (k / dim);
    }
  }
}

void make_seps (void) {
  for (int cl = 0; cl < CLASSES; cl++) {
    int serial = 0;
    seps[cl].reset();
    for (int i = 0; i < REPS; i++) {
      for (int j = i + 1; j < REPS; j++) {
	assert(serial < seps[cl].size());
	seps[cl].set(serial++, table[i][cl] == table[j][cl]);
      }
    }
  }
}

void find_sep_set_2 (void) {
  for (int a = 0; a < CLASSES; a++) {
    for (int b = a+1; b < CLASSES; b++) {
      bs x = seps[a] & seps[b];
      if (x.none()) {
	printf("Found %d,%d\n",a+1,b+1);
	//cout << x << endl;
      }
    }
  }
}

// brute force for a 3-size set

void find_sep_set_3 (void) {
  for (int a = 0; a < CLASSES; a++) {
    for (int b = a+1; b < CLASSES; b++) {
      for (int c = b+1; c < CLASSES; c++) {
	bs x = seps[a] & seps[b] & seps[c];
	if (x.none()) {
	  printf("Found %d,%d,%d\n",a+1,b+1,c+1);
	  //cout << x << endl;
	}
      }
    }
  }
}

void find_sep_set_4 (void) {
  int i = 0;
  for (int a = 0; a < CLASSES; a++) {
    for (int b = a+1; b < CLASSES; b++) {
      for (int c = b+1; c < CLASSES; c++) {
	for (int d = c+1; d < CLASSES; d++) {
	  bs x = seps[a] & seps[b] & seps[c] & seps[d];
	  if (x.none()) {
	    printf("$\\{ %d,%d,%d,%d \\}$",a+1,b+1,c+1,d+1);
	    if (++i % 4 == 0)
	      printf(" \\\\ \\hline\n");
	    else
	      printf(" &\n");
	  }
	}
      }
    }
  }
}

bs already_got;

bool bs_comp(const bs *a, const bs *b) {
  if (!a)
    return false;
  if (!b)
    return true;
  bs aa = ~*a & already_got;
  bs bb = ~*b & already_got;
  return aa.count() > bb.count();
}



int greedy(void) {
  bs *s[CLASSES];
  int sepset[CLASSES];
  int sepsetsize = 0;
  int size = 0;
  for (int i = 0; i < CLASSES; i++) s[i] = &seps[i];
  already_got.set();
  while (already_got.any()) {
    sort(s, s + CLASSES, bs_comp);
    assert(s[0]);
    bs **p = &s[0];
    while (!*p) p++;
    
    already_got &= **p;
    sepset[sepsetsize++] = *p - seps;
    printf("Using number %d\n", *p - seps + 1);
    cout << "Added\t\t" << **p << endl;
    cout << "Now have\t" << already_got << endl;
    cout << "Any left:\t" << already_got.any() << endl;
    *p = NULL;
    size++;
  }
  printf("Found separating set of size %d:", size);
  for (int j = 0; j < sepsetsize; j++)
    printf(" %d", sepset[j]+1);
  printf("\nRelevant columns:\n");
  for (int i = 0; i < REPS; i++) {
    for (int j = 0; j < sepsetsize; j++)
      printf("%2d ", table[i][sepset[j]]);
    printf("\n");
  }
  return sepsetsize;
}  

int main(void) {
  get_table();
#ifndef QUIET
  for (int i = 0; i < REPS; i++) {
    for (int j = 0; j < CLASSES; j++)
      printf("%2d ", table[i][j]);
    printf("\n");
  }
  printf("\n");
#endif
  make_eigenvalues();
#ifndef QUIET
  for (int i = 0; i < REPS; i++) {
    for (int j = 0; j < CLASSES; j++)
      printf("%2d ", table[i][j]);
    printf("\n");
  }
#endif
  make_seps();
  for (int cl = 0; cl < CLASSES; cl++) {
    //cout << cl << ":\t\t" << seps[cl] << endl;
  }
  find_sep_set_2();
  //greedy();
  return 0;
}
