#include <assert.h>
#include <vector>
#include "wreath.h"

const int primes[] = { 
  2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,
  61,67,71,73,79,83,89,97,101,103,107,109,113,127,
  131,137,139,149,151,157,163,167,173,179,181,191,
  193,197,199
};

const int nprimes = sizeof(primes);
const int maxprime = 199;

static vector<int> mu_values;

void compute_mus(int maxn)
{
  for (int n = mu_values.size(); n <= maxn; n++)
    mu_values.push_back(compute_mu(n));
}

int mu(int n)
{
  compute_mus(n);
  return mu_values[n];
}


int compute_mu(int n)
{
  assert(n <= maxprime);
  int t = 0;
  for (int i = 0; primes[i] <= n; i++)
    {
      if (n % primes[i] == 0)
	{
	  n /= primes[i];
	  if (n % primes[i] == 0) // has two factors of primes[i]
	    return 0;
	  else
	    t++;
	}
    }
  return (t % 2 == 0) ? 1 : -1;
}

bool isprime(int n)
{
  switch (n) {
  case 2:
  case 3:
  case 5:
  case 7:
  case 11:
  case 13:
  case 17:
  case 19:
  case 23:
  case 29:
  case 31:
  case 37:
    return true;
  default:
    if (n % 2 == 0)
      return false;
    for (int i = 3; i*i <= n; i += 2) {
      if (n % i == 0)
	return false;
    }
    return true;
  }
}

int gcd(int a, int b) {
  if (b == 0)
    return a;
  else
    return gcd(b, a % b);
}

int lcm(int a, int b) {
  return a * b / gcd(a,b);
}

